from t5html.lineparser import *

import pytest
from textwrap import dedent
from collections import namedtuple


class TestLineClassifiers:
    def test_classify_comment_line(s):
        assert classify_line('## Some comment') == "comment"

    def test_classify_continue_line(s):
        assert classify_line('.. follow up') == "continue"

    def test_classify_verbatim_line(s):
        assert classify_line('!! <!- html comment -->') == "verbatim"
        
    def test_classify_element_line(s):
        assert classify_line('      div#gallery.blog') == "element"

    def test_classify_import_line(s):
        assert classify_line('@ path/to/file') == "import"


tdata_indent = [ ('', 0), (' '*3, 1), (' '*6, 2), (' '*7, 2), ('\t'*3, 0) ]
@pytest.mark.parametrize("input, expected", tdata_indent)
def test_indent_level(input, expected):
    assert get_indent_level(input) == expected


class TestLineParsing:
    """
    # t5html

    HTML5 := <!DOCTYPE html>
    !! HTML5
    @@ file from src

    html
       head
          title > "t5html example
       body
       header | nav
       main
          article
             section
                "text node
                .. over multiple line
          article
             section
    """
    def setup_class(c):
        c._html = dedent(c.__doc__)

    def test_doc_as_text(s):
        assert '# t5html' in s._html

    def test_indentation(s):
        body = [l for l in s._html.splitlines() if 'body' in l][0]
        title = [l for l in s._html.splitlines() if 'title' in l][0]
        assert 'body' in body and get_indent_level(body) == 1
        assert 'title' in title and get_indent_level(title) == 2

    def test_classify_lines(s):
        b, c, n, m, v, i, cc, t =\
        "blank comment element macro verbatim import continue text".split()
        manual = enumerate([b,c,b,m,v,i,b,*(n,)*8,t,cc,n,n])
        auto = enumerate(map(classify_line, s._html.splitlines()))
        assert list(auto) == list(manual)

    def test_RawLineStructure_fromRawString(s):
        rls = RawLines_from_str(s._html)
        assert rls[1] == (1, '# t5html')
        assert rls[-1] == (18, ' '*9+'section')

    def test_classifed_LineStructure(s):
        cls = LineStructures_from_RawLines(
                RawLines_from_str(s._html))
        assert cls[1] == (1, '# t5html', 'comment')
        assert cls[-1] == (18, ' '*9+'section', 'element')

    def test_sanitized_cls(s):
        scls = sanitized_LineStructures(
                LineStructures_from_RawLines(
                    RawLines_from_str(s._html)))
        assert scls[0] == (3, 'HTML5 := <!DOCTYPE html>', 'macro')
        assert scls[1] == (4, '!! HTML5', 'verbatim')
        assert scls[2] == (5, '@@ file from src', 'import')
        assert scls[-3] == (16, ' '*12+'.. over multiple line', 'continue')
        assert scls[-1] == (18, ' '*9+'section', 'element')

    
class TestSanitizedLineParsing:
    """
    # t5html

    HTML5 := <!DOCTYPE html>
    !! HTML5
    @@ file from src

    html
       head
          title > "t5html example
       body
       header | nav
       main
          article
             section
                "text node
                .. over multiple line
          article
             section
    """
    def setup_class(c):
        c._html = dedent(c.__doc__)
        c._scls = sanitized_LineStructures(
                    LineStructures_from_RawLines(
                        RawLines_from_str(c._html)))

    def test_extract_macros(s):
        macros, cls = split_macros(s._scls)
        assert macros == [(3, 'HTML5 := <!DOCTYPE html>', 'macro')]
        assert (4, '!! HTML5', 'verbatim') in cls
        assert (5, '@@ file from src', 'import') in cls

    def test_extract_imports(s):
        imports, cls = split_imports(s._scls)
        assert imports == [(5, '@@ file from src', 'import')]
        assert (3, 'HTML5 := <!DOCTYPE html>', 'macro') in cls
        assert (4, '!! HTML5', 'verbatim') in cls

    def test_macrodef_from_LineStructure(s):
        macros, cls = split_macros(s._scls)
        assert MacroDef_from_LineStructures(macros) == {'HTML5' : '<!DOCTYPE html>'}

    def test_expand_macro(s):
        macros, cls = split_macros(s._scls)
        macrodef = MacroDef_from_LineStructures(macros)
        assert '!! <!DOCTYPE html>' in [tpl.line for tpl in expand_macros(cls, macrodef)]

    def test_concatenate_lines_lessmocked(s):
        lines = content_from_ls(concatenate_lines(s._scls))
        assert ' '*12+'"text node over multiple line' in lines
        lines = concatenate_lines(s._scls)
        assert (15, ' '*12+'"text node over multiple line', 'text') in lines

    def test_concatenate_lines_mocked(s):
        lines = [LineStructure(0, '"Text Node', 'text'),
                LineStructure(1, '.. over multiple lines.', 'continue')]
        assert '"Text Node over multiple lines.' in concatenate_lines(lines)[0].line
        lines = [LineStructure(0, '"Text Node', 'text'),
                LineStructure(1, '..over multiple lines.', 'continue')]
        assert '"Text Nodeover multiple lines.' in concatenate_lines(lines)[0].line
        lines = [LineStructure(0, '"Text Node', 'text'),
                LineStructure(1, '.. over multiple lines.', 'continue')]
        assert [LineStructure(0, '"Text Node over multiple lines.', 'text'),
                ] == concatenate_lines(lines)

    def test_split_fold(s):
        foldseq = list(">|<")
        line = "p > c | s < p"
        assert split_fold(line, foldseq) == ['p', 'c', 's', 'p']

    def test_line_folding_1(s):
        line = "parent > child "
        assert fold_lines([LineStructure(0, line, 'element')]) == [
                LineStructure(0, 'parent', 'element'),
                LineStructure(0, '   child ', 'element')]

    def test_line_folding_2(s):
        line = "parent | sibling"
        assert fold_lines([LineStructure(0, line, 'element')]) == [
                LineStructure(0, 'parent', 'element'),
                LineStructure(0, 'sibling', 'element')]

    def test_line_folding_3(s):
        line = "parent > child | sibling"
        assert fold_lines([LineStructure(0, line, 'element')]) == [
                LineStructure(0, 'parent', 'element'),
                LineStructure(0, '   child', 'element'),
                LineStructure(0, '   sibling', 'element')]

    def test_line_folding_4(s):
        line = "parent > child | sibling < parent"
        assert fold_lines([LineStructure(0, line, 'element')]) == [
                LineStructure(0, 'parent', 'element'),
                LineStructure(0, '   child', 'element'),
                LineStructure(0, '   sibling', 'element'),
                LineStructure(0, 'parent', 'element')]

    def test_line_folding_5(s):
        "root element is already indented"
        line = "   parent > child | sibling < parent"
        assert fold_lines([LineStructure(0, line, 'element')]) == [
                LineStructure(0, '   parent', 'element'),
                LineStructure(0, '      child', 'element'),
                LineStructure(0, '      sibling', 'element'),
                LineStructure(0, '   parent', 'element')]



# vi: set et ts=4 ts=4 ai cc=78 nowrap rnu so=5:



""" 
LineParser

Contains the functions to process t5html files.

Example:

    Start of line symbols:

        # comment
        ! verbatim line
        @ import file
        MACRONAME := macro definition / text
        .. line continuation
    
    Inline symbols:

        > indent
        < dedent
        | keep level (sibling element to previous element)
        " text-node

"""
from collections import namedtuple, OrderedDict
import re


## as we progress from raw-text to html, the data structures 
## continually evolve. Every line itself is structured by
## (original line-number, content of the line)
## the next step is to add a classifier to the structure:
## (original line-number, content of the line, classifier)

RawLine = namedtuple('RawLine', 'nr line')
LineStructure = namedtuple('LineStructure', 'nr line cls')


# read lines
# remove comments and empty lines
# extract macro and import lines
# ignore verbatim lines
# expand macros (therefore macros could contain reformating symbols)
# concatenate/split inline > < and | smybols


def get_indent_level(line):
    """
    takes a string representing a line 
    returns a number representing the indentation-level
    """
    # Because of the line-continuation symbol beeing two dots followed
    # by a space, the *reasonable* indentation seems to be 3.
    # explicit only element whitespaces! no tabs, etc.
    count = len(line) - len(line.lstrip(' '))
    # TODO: we don't handle indentation errors, atm!
    level = int(count / 3)
    return level


def RawLines_from_str(text):
    """
    takes a string 
    returns a RawLineStructure [(nr, line), ...]
    """
    rls = [RawLine(n, l.rstrip())
            for n, l in enumerate(text.splitlines())]
    return rls


def LineStructures_from_RawLines(rls):
    """
    tales a RawLinesStructure 
    returns a LineStructuresStructure
    """
    cls = [LineStructure(n, l, classify_line(l))
            for n, l in rls]
    return cls


def sanitized_LineStructures(cls):
    """
    takes a LineStructureStructure
    returns a CLS without blanks and comments
    """
    sls = [t for t in cls if t.cls not in ('comment', 'blank')]
    return sls


def split_by_classifier(cls, clsname):
    """
    takes a LineStructuresStructure 
    returns a split by named classifier
    """
    extracted = [t for t in cls if t.cls == clsname] or []
    rest = [t for t in cls if not t.cls == clsname]
    return extracted, rest


def split_macros(scls):
    """
    takes a LineStructuresStructure
    returns two lists: (macros, scls without macros)
    """
    return split_by_classifier(scls, 'macro')
    

def split_imports(scls):
    """
    takes a LineStructuresStructure
    returns two lists: (imports, scls without imports)
    """
    return split_by_classifier(scls, 'import')
    

def classify_line(line):
    """
    takes a str representing a line
    returns a string classifying the type of line

    atm, there are 8 line-cllassifiers:
        blank, comment, continue, element, import, macro, text, verbatim
    """
    l = line.strip()
    # single classifier allowed:
    if l == '': return 'blank'

    if l.startswith('#'): return "comment"
    if l.startswith('!'): return "verbatim"
    if l.startswith('@'): return "import"
    if l.startswith('"'): return "text"

    # has to be a double-classifier
    if l.startswith('..'): return "continue"
    if ' := ' in l and l.split(' := ', 1)[0].isupper():
        return 'macro'

    return "element"


def MacroDef_from_LineStructures(cls):
    """
    takes classifiedLines
    returns a OrderedDict of MacorKey: MacroValue
    """
    macrodef = OrderedDict()
    for m in cls:
        k, v = m.line.split(' := ')
        macrodef[k] = v
    return macrodef
    

def Imports_from_LineStructures(cls):
    """
    takes classifiedLines
    returns a OrderedDict of MacorKey: MacroValue
    """
    imports = OrderedDict()
    for i in cls:
        k, v = i.line.lstrip('@ ')
        imports[k] = v
    return imports


def expand_macros(cls, macros):
    """
    takes LineStructures
    returns LineStructures, but without macros.
    """
    macro_free = []
    for ls in cls:          # ls = line structure (nr, line, cls)
        line = ls.line
        for m in macros:
            if m in line:
                line = ls.line.replace(m, macros[m])
                
        macro_free.append(LineStructure(ls.nr, line, ls.cls))
        
    return macro_free if macro_free else cls


def concatenate_lines(ls):
    """
    takes a list of LineStructures and 
    returns a list of concatenated LineStructures, without 'continue' lines
    """
    concatenated = []
    for current in ls:
        new = current
        if current.cls == 'continue':
            lastline = concatenated.pop()
            new_content = lastline.line + current.line.lstrip().lstrip('.')
            new = LineStructure(lastline.nr,
                                new_content,
                                lastline.cls)
        concatenated.append(new)

    return concatenated

FOLDS = re.compile(r' (<|>|\|) ')
def fold_lines(ls):
    """
    takes a list of  LineStructures
    returns a list of LineStructures
    """
    folded = []
    for current in ls:
        foldseq = FOLDS.findall(current.line)
        if not foldseq:
            folded.append(current)
            continue

        lines = split_fold(current.line, foldseq)
        level = get_indent_level(lines[0])
        for idx, fold in enumerate(foldseq, start=1):
            level = level + 1 if fold == '>' else level
            level = level - 1 if fold == '<' else level
            level = level if level > 0 else 0
            indent = (3 * level) * ' '
            lines[idx] = indent + lines[idx]

        for line in lines:
            folded.append(LineStructure(current.nr, line, current.cls))
    return folded


def split_fold(linestr, foldseq):
    """
    takes a line as string and a sequence of fold-symbols to
    return a list of lines splitted by the fold-symbols
    """
    folded, line = [], linestr
    for sym in foldseq:
        line, nextline = line.split(sym, 1)
        folded.append(line.rstrip())
        line = nextline.lstrip()
    else:
        folded.append(line.lstrip())
    return folded


def parse_str(t5html):
    """
    takes a raw string formatted as t5html-text
    returns a completly processed list of LineStrucrues ready to be converted
        to html
    """
    raw = RawLines_from_str(t5html)
    lines = LineStructures_from_RawLines(raw)
    lines = sanitized_LineStructures(lines)
    lines = concatenate_lines(lines)
    macros, lines = split_macros(lines)
    imports, lines = split_imports(lines)
    macrodef = MacroDef_from_LineStructures(macros)
    lines = expand_macros(lines, macrodef)
    lines = fold_lines(lines)
    # BUG: 2nd expansion needed because of a folding/macro issue
    lines = expand_macros(lines, macrodef)

    # reclassify the lines after all the folding and unfolding:
    lines = [LineStructure(l.nr, l.line, classify_line(l.line)) for l in lines]
    return lines


def content_from_ls(ls):
    return [l.line for l in ls]
    

if __name__ == '__main__':
    print("This file is meant to be imported, not to be executed.")


# vi: set et ts=4 ts=4 ai cc=78 rnu so=5 nuw=4:



""" 
ElementParser

Contains the functions to process t5html formatted element-lines aka tags.

Example:
    
    parse_element('section#overview.blog-content.top-entry breaking')
    
    returns:

        '<section id="overview" class="blog-content top-entry" breaking>'

"""

def parse_element(line):
    """
    takes a raw-line and
    returns a string formatted as a html-tag
    """
    element, attributes = separate_element_from_attributes(line)
    _es = elementStructure_fromString(element)
    _as = attributesStructure_fromString(attributes)

    element_head = stringify_ElementStructure(_es)
    element_attributes = stringify_AttributeStructure(_as)

    line = f'<{element_head} {element_attributes}>' if element_attributes else f'<{element_head}>'
    return line


def element_name(elementstr):
    """
    takes a string formatted as a html-tag
        e.g.: '<div id="example">
    returns a string with the name of the tag
        e.g.: 'div'
    """
    # There's no error checking!
    return elementstr[1:-1].split(' ', 1)[0]


def separate_element_from_attributes(line):
    """
    takes a raw element-line (e.g.: 'ele#id.cls atr1 atr2=val') and
    returns tuple(element, attributes) (e.g.: ('ele#id.cls', 'atr1 atr2=val'))
    """
    element, attributes = line.split(' ', 1) if ' ' in line else (line, '')
    return element, attributes


def elementStructure_fromString(element):
    """
    tales an element (e.g.: 'ele#id.cls.cls.cls') and
    returns a (element, id, [class,]) tuple
    """
    classes = [c for c in element.split('.') if '.' in element]
    element_id = classes.pop(0) if classes else element
    element, _id = element_id.split('#') if '#' in element_id else (element_id, '')

    return element, _id, classes


def attributesStructure_fromString(attribute_str):
    """
    takes a string of attributes and 
    returns a list of attribute-tuples
    """
    # There are three possible 'words':
    #   1. standalone attributes (aka bool. attr), 2a. simple assignments (key=value), 
    #   2b. text assignments (key="quoted longer text")

    words = attribute_str.split()
    next_word_belongs_to_last_value = False
    attributes = []

    for word in words:

        if next_word_belongs_to_last_value:

            # remove last (key, value) pair and add current word to value
            # reappend and get next word
            
            (key, value) = attributes.pop()
            if word.endswith('"'):
                next_word_belongs_to_last_value = False
            value += ' ' + word.strip('"')
            attr = (key, value)
            
            attributes.append(attr)
            continue

        is_assignment = '=' in word
        if is_assignment:
            key, value = word.split('=', 1)
            if value.startswith('"') and not value.endswith('"'):
                next_word_belongs_to_last_value = True
            attr = key, value.strip('"')
        else:
            # boolean attribute
            attr = (None, word)

        attributes.append(attr)

    return attributes


def stringify_AttributeStructure(attrlst):
    """
    takes a list of attribute tuples (key, value) or (None, boolean-attr) and
    returns a string

    """
    reformated = [f'{key}="{value}"' if key else f'{value}' for (key, value) in attrlst]
    return ' '.join(reformated)


def stringify_ElementStructure(es):
    """
    takes a element tuple (tag, id, [classes]) and
    returns a string
    """
    (tag, _id, classes) = es
    line = f'{tag} id="{_id}"' if _id else f'{tag}'
    line += f' class="{" ".join(classes)}"'  if classes else ''
    
    return line


if __name__ == '__main__':
    print("This file is meant to be imported, not to be executed.")


# vi: set et ts=4 ts=4 ai cc=78 rnu so=5 nuw=4:



import pytest

from t5html.elementparser import *


class Test_Element_Attr_Separation:
    """ 
    Split Elements/Tags with their id and class attributes from the other
    more specific attributes.
    """
    f = staticmethod(separate_element_from_attributes)
    def test_element(s):
        assert s.f('ele') == ('ele', '')

    def test_element_id(s):
        assert s.f('ele#id') == ('ele#id', '')

    def test_element_id_cls(s):
        assert s.f('ele#id.cls') == ('ele#id.cls', '')

    def test_element_cls(s):
        assert s.f('ele.cls') == ('ele.cls', '')

    def test_element_id_2cls(s):
        assert s.f('ele#id.cls.cls') == ('ele#id.cls.cls', '')

    def test_element_2cls(s):
        assert s.f('ele.cls.cls') == ('ele.cls.cls', '')


class Test_Element_Id_Class_Separation:
    """
    Check if there are besides the tag additional id and class attributes.
    """
    f = staticmethod(elementStructure_fromString)
    def test_element(s):
        assert s.f('ele') == ('ele', '', [])

    def test_element_id(s):
        assert s.f('ele#id') == ('ele', 'id', [])

    def test_element_id_cls(s):
        assert s.f('ele#id.cls') == ('ele', 'id', ['cls'])

    def test_element_cls(s):
        assert s.f('ele.cls') == ('ele', '', ['cls'])

    def test_element_id_2cls(s):
        assert s.f('ele#id.cls.cls') == ('ele', 'id', ['cls', 'cls'])

    def test_element_2cls(s):
        assert s.f('ele.cls.cls') == ('ele', '', ['cls', 'cls'])


class Test_Attribute_Conversion:
    """
    Check for normal attribute-str to list conversion.
    """
    f = staticmethod(attributesStructure_fromString)
    def test_attr(s):
        assert s.f('attr') == [(None, 'attr')]
    
    def test_attr_val(s):
        assert s.f('attr=val') == [('attr', 'val')]
    
    def test_2attr(s):
        assert s.f('attr attr') == [(None, 'attr'), (None, 'attr')]

    def test_attr_val_attr(s):
        assert s.f('attr=val attr') == [('attr','val'), (None, 'attr')]

    def test_attr_val_x2(s):
        assert s.f('attr=val attr=val') == [('attr', 'val'), ('attr', 'val')]

    def test_attr_val_2x_quoted(s):
        assert s.f('attr="val" attr=val') == [('attr','val'), ('attr', 'val')]

    def test_attr_namespace(s):
        assert s.f('p:attr') == [(None, 'p:attr')]
    
    def test_attr_ns_val(s):
        assert s.f('p:attr=val') == [('p:attr', 'val')]
    
    def test_attr_text_assignment(s):
        assert s.f('attr="Some text"') == [('attr','Some text')]


class Test_String_From_Attribute_Tuples:
    """
    Check for normal attribute-str to list conversion.
    """
    f = staticmethod(stringify_AttributeStructure)
    def test_string_from_attribute_tuples(s):
        assert s.f([('attr', 'Some text')]) == 'attr="Some text"'

    def test_string_from_attribute_tuples_boolattr(s):
        assert s.f([(None, 'attr')]) == 'attr'

    def test_string_from_attribute_tuples_2boolattr(s):
        assert s.f([(None, 'attr'), (None, 'attr')]) == 'attr attr'

    def test_string_from_attribute_tuples_complex(s):
        _in = [("key", "value"), ("k", 'longer text'), (None, 'attr')]
        _out = 'key="value" k="longer text" attr'
        assert s.f(_in) == _out


class TestComplexAttributeExtraction:
    """
    check for both attribute and tag extraction
    """
    def test_long_element(s):
        _in = 'tag#id.cls.dls.els attr attr=ab attr="abc def" attr=last'
        _out = 'attr attr="ab" attr="abc def" attr="last"'
        ele, attrs = separate_element_from_attributes(_in)
        tpls = attributesStructure_fromString(attrs)
        assert stringify_AttributeStructure(tpls) == _out
        _out = ('tag', 'id', ['cls', 'dls', 'els'])
        assert elementStructure_fromString(ele) == _out


class TestStringifyElementTuple:
    """
    tag head to string
    """
    f = staticmethod(elementStructure_fromString)
    def test_long_element(s):
        _in = s.f('tag#id.cls.dls.els')
        _out = 'tag id="id" class="cls dls els"'
        assert stringify_ElementStructure(_in) == _out


class TestReformatElementLine:
    def test_parse_element(s):
        _in = 'div#myid.cls.cls attr1=value'
        _out = '<div id="myid" class="cls cls" attr1="value">'
        assert parse_element(_in) == _out

    def test_get_tag(s):
        _in = 'div#myid.cls.cls attr1=value'
        _out = '<div id="myid" class="cls cls" attr1="value">'
        assert element_name(parse_element(_in)) == 'div'


# vi: set et ts=4 ts=4 ai cc=78 nowrap nu so=5:



""" 
TreeBuilder

Creates a HTML-File from a t5html formatted file.
Utilizes the lineparser and elementparser modules.

Example:

    html = HTML_from_t5html(rawstr)

"""

from . import lineparser as lp
from . import elementparser as ep

import sys
from collections import namedtuple
from itertools import zip_longest


TreeElement = namedtuple("TreeElement", "level tag value orig_lnr")


def build_from_str(t5htmlstr):
    """
    takes a rawstr in t5html-format
    returns a list of LineStructures
    """
    return lp.parse_str(t5htmlstr)


def LineStructureFactory(src):
    """
    takes either a string or a list of LineStructures
    returns a list of LineStructures

    basically ensures that we have LineStructures independently if we are 
    fed a string in t5html format or already a converted list of
    LineStructures.
    """
    if type(src) == str:
        return build_from_str(src)
    elif type(src) == list and src and type(src[0]) == lp.LineStructure:
        # non empty list (`and src`) with 1st item of type LineStructure
        return src


def pseudoAST_from(structured_lines):
    """
    tales a list of linestructures
    returns a pseudoast
    """
    getvalue = lambda l: l.line.lstrip()\
                            if l.cls != 'element'\
                            else ep.parse_element(l.line.lstrip())
    return [ TreeElement(
                lp.get_indent_level(line.line),
                ep.element_name(getvalue(line)) if line.cls == 'element' else None,
                getvalue(line), 
                line.nr) for line in structured_lines]


def html_tree_from_ast(ast):
    """
    takes a (pseudo-)ast
    returns a list of formatted html lines
    """
    indentstr = lambda x: x*2*" "
    tree, tagstack = [], []
    for element, peek in zip_longest(ast, ast[1:], fillvalue=None):
        # this will cause errors for text-nodes with values like: "! some text 
        line = element.value.strip('"').strip('!') if not element.tag else element.value
        level = element.level
        if peek:
            if peek.level > element.level:
                # normal tag
                tagstack.append(element.tag)
            elif peek.level == element.level and element.tag:
                # self-closing tag
                line = line[:-1] + '/>'
        tree.append(indentstr(level) + line)

        if peek and peek.level < element.level:
            level = peek.level
            line = '</' + tagstack.pop() + '>' if tagstack else ''
            tree.append(indentstr(level) + line)
    else:
        while tagstack:
            line = '</' + tagstack.pop() + '>' if tagstack else ''
            level -= 1
            tree.append(indentstr(level) + line)
    return tree


def Tree_from(src):
    """
    takes an input 
    returns a pseudo-ast 
    """
    structured_lines = LineStructureFactory(src)
    if not structured_lines:
        # probably better in LineStructureFactory. E.g.: if there's a list of
        # non LineStructures, to differentiate from a valid LineStructures-List
        raise Exception("Can't parse source of type: %s." % type(src))
    ast = pseudoAST_from(structured_lines)
    tree = html_tree_from_ast(ast)
    return tree


def HTML_from_t5html(src):
    """
    takes a string in t5html format
    returns a string as HTML5
    """
    return '\n'.join(Tree_from(src)).strip()
    

if __name__ == '__main__':
    print("This file is meant to be imported, not to be executed.")


# vi: set et ts=4 ts=4 ai cc=78 rnu so=5 nuw=4:



from t5html.treebuilder import *
from t5html.lineparser import *


import pytest

@pytest.mark.parametrize("src, expected",
                            [('', "Can't parse source of type"),
                             (0, "Can't parse source of type"),
                             ([], "Can't parse source of type"),
                           ])
def test_Tree_from_exception_on_wrong_input_type(src, expected):
    with pytest.raises(Exception) as einfo:
        b = Tree_from(src)
    assert str(einfo.value).startswith(expected)

@pytest.mark.parametrize("src, expected",
        [(pseudoAST_from([LineStructure(0, '!! TEST', 'verbatim')]), list),
         (pseudoAST_from([LineStructure(0, '!! TEST', 'verbatim')])[0], TreeElement)])
def test_pseudoAST_exception_on_correct_input(src, expected):
    assert type(src) == expected

def test_pseudoAST_simple_t5html_input():
    data = 'html > head | body > div#main.imp.test > p > "some text'
    tree = pseudoAST_from(LineStructureFactory(data))
    assert TreeElement(3,'p','<p>',0) in tree
    assert TreeElement(1,'body','<body>',0) in tree

def test_html_from_t5html():
    data = 'html > head | body > div#main.imp.test > p > "some text'
    assert '  </body>' in HTML_from_t5html(data) 

# vi: set et ts=4 ts=4 ai cc=78 :



import pytest

from os import path, listdir
from glob import glob

def test_docs_exist():
    """ make sure we have documentation """ 
    assert path.isdir('doc')

def test_docs_non_empty():
    """doc dir is not empty"""
    assert len(listdir('doc')) > 0

def test_doc_readme_exists():
    """at least 1 readme file"""
    assert len(glob('doc/ReadMe*')) > 0
    

# vi: set et ts=4 ts=4 ai :



#!/usr/bin/python3
"""
t5html converts a t5html-formatted file into HTML

USAGE:

    t5html filename
"""
from . import make_html

import sys

example = """
## t5html

DOCTYPE := <!DOCTYPE html>

!! DOCTYPE

!! <-- This is
.. a html comment
.. over multiple lines -->

html
   head
   body
      div#id.cls1.cls2 attr1=value1 attr2="some quoted value"
         article#id > p > "textnode
            .. over multiple
            .. lines
            section
               p#id1 | p#id2 | p#id3
               p#id4 > "Text Node < p#id5
               p#id6
                  "Text Node
"""

def start():
    """
    entry point
    """
    if len(sys.argv) == 2:
        fname = sys.argv[1]
    else:
        exit(__doc__ + "\n\nERROR: Seems like we didn't get a filename")
    with open(fname) as f:
        input = f.read()
    print(make_html(input))


if __name__ == "__main__":
    start()


# vi: set et ts=4 ts=4 ai cc=78 nowrap nu so=5:



#!/usr/bin/python3
""""
t5html-lib contains the function to process a t5html formatted file.

the main-function is make_html.

Example:

    from htmllib import make_html

    t5html = open("example.t5h").read()
    html = make_html(t5html)

"""
from .treebuilder import HTML_from_t5html as make_html
